==> speakrec-0.1.4.tar.gz: speakrec-0.1.4/menu.cpp <==
/*
 * SpeakRec - Menu
 * Copyright  2003 Schou Industries ApS.
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "speakrec.h"
#include "menu.h"

/*
 * Define menu function prototype.
 */
#define M_F (char *(*)(struct menuset *menu_entry))

struct menuset {
	menu_id id;
	menu_id parent;
	char *command;
	char *(*func)(struct menuset *menu_entry);
	char *name;
	char *desc;
};

char *currentname(void) {
	return "1234";
}

char system_command[256];

extern int opt_lf30hz;
extern off_t counter_index;
extern off_t counter_size;

const char *menu_lf30hz[] = {
	"Confirm turn lf30hz on",
	"Confirm turn lf30hz off"
};

char *act_rstcnt(struct menuset *menu_entry) {

	counter_index = counter_size;
	menu_entry->desc = (char *)"Reset counter";
	menu_entry->name = (char *)"Rst Cnt";

	return 0;
}

char *act_lf30hz(struct menuset *menu_entry) {

	opt_lf30hz++;
	opt_lf30hz %= 2;
	menu_entry->desc = (char *)menu_lf30hz[opt_lf30hz];
	menu_entry->name = (char *)(opt_lf30hz ? "Cnf off" : "Cnf  on");

	return 0;
}

char *act_system(struct menuset *menu_entry) {
	int ret;

	snprintf(system_command, sizeof(system_command)-1, "%s.%s", PACKAGE, menu_entry->command);
	ret = system(system_command);
	snprintf((char *)&system_command[strlen(system_command)],
			sizeof(system_command)-strlen(system_command),
			" = %d (return code on execute)", ret);

	return system_command;
}

char *act_delall(struct menuset *menu_entry) {
	counter_index = 0;
	return act_system(menu_entry);
}

struct menuset menu_item[] = {
/*
 * id            parent    command   func            name       desc
 */
{ M_RSTCNT,      M_ROOT,   "rstcnt", M_F act_rstcnt, "RST CNT", "Reset time counter" },
{ M_DELALL,      M_ROOT,   NULL,     NULL,           "Del All", "Delete all files" },
//{ M_DELALL_CONF, M_DELALL, "delall", M_F act_system, "Confirm", "Confirm delete all files" },
{ M_DELALL_CONF, M_DELALL, "delall", M_F act_delall, "Confirm", "Confirm delete all files" },
/*
{ M_NET,         M_ROOT,   NULL,     NULL,           "  Net",   "Upload files to server over net" },
{ M_NET_CONF,    M_NET,    "net",    M_F act_system, "Confirm", "Confirm upload files" },
{ M_CDREC,       M_ROOT,   NULL,     NULL,           "Cd Rec",  "Record (burn) a CD-ROM" },
{ M_CDREC_AUDIO, M_CDREC,  "cdra",   M_F act_system, "Audio",   "Audio (music) CD-ROM (stereo)" },
{ M_CDREC_WAV,   M_CDREC,  "cdrw",   M_F act_system, "Wave",    "Data CD-ROM with wav-files (mono)" },
{ M_CDREC_OGG,   M_CDREC,  "cdro",   M_F act_system, "Ogg Vor", "Data CD-ROM with Ogg/Vorbis-files (mono)" },
{ M_CDREC_SPX,   M_CDREC,  "cdrs",   M_F act_system, "Speex",   "Data CD-ROM with Speex-files (mono)" },
{ M_CDREC_MP3,   M_CDREC,  NULL,     NULL,           "MP3",     "Data CD-ROM with mp3-files - Currently disabled due to patent infringement" },
*/
{ M_COPYRIGHT,   M_ROOT,   "cpyrght",M_F act_system, "Cpyrght", "SpeakRec ver " VERSION " Copyright " YEAR " Schou Industries ApS and Speltron.dk" },
{ M_LF30HZ,      M_ROOT,   NULL,     NULL,           "LF30Hz",  "Toggle LF30Hz on/off" },
{ M_LF30HZ_CONF, M_LF30HZ, "lf30hz", M_F act_lf30hz, "Cnf  on", "Confirm on" },
{ M_REBOOT,      M_ROOT,   NULL,     NULL,           "Reboot",  "Reboot the system and start again" },
{ M_REBOOT_CONF, M_REBOOT, "reboot", M_F act_system, "Confirm", "Confirm rebooting system" },
{ M_HALT,        M_ROOT,   NULL,     NULL,           "Halt",    "Halt the system and turn power off" },
{ M_HALT_CONF,   M_HALT,   "halt",   M_F act_system, "Confirm", "Confirm halting system" }
};

#define SIZE_MENU (sizeof(menu_item) / sizeof(menu_item[0]))

#define inc_index(i) ((i + 1) % SIZE_MENU)
#define dec_index(i) ((i - 1 + SIZE_MENU) % SIZE_MENU)

CMenu::CMenu(char *newcurrentfile) {

	currentfile = newcurrentfile;
	idx = 0;
	settext(idx);

}

CMenu::~CMenu(void) {
}

int CMenu::settext(int index) {
	menu_id tmp_id;
	char *desc;

	current_tcount = 0;
	current_text[current_tcount++] = menu_item[index].name;
	desc = menu_item[index].desc;
	while (menu_item[index].parent != M_ROOT) {
		tmp_id = menu_item[index].parent;
		while (tmp_id != menu_item[index].id) {
			index = inc_index(index);
		}
		current_text[current_tcount++] = menu_item[index].name;
	}
	current_text[current_tcount++] = desc;
	current_text[current_tcount++] = "Press play for enter";
	current_text[current_tcount++] = "Press menu for exit";

	return index;
}

int CMenu::up(int index) {
	menu_id tmp_id;

	tmp_id = menu_item[index].parent;
	while (tmp_id != menu_item[index].id) {
		index = inc_index(index);
	}
	settext(index);

	return index;
}

int CMenu::down(int index) {
	int i;

	i = inc_index(index);
	while (i != index && menu_item[i].parent != menu_item[index].id) {
		i = inc_index(i);
	}
	settext(i);

	return i;
}

int CMenu::next(int index, int forward) {
	menu_id tmp_id;

	tmp_id = menu_item[index].parent;
	do {
		index = forward ? inc_index(index) : dec_index(index);
	} while (tmp_id != menu_item[index].parent);
	settext(index);
	
	return index;
}

int CMenu::do_action(tape_action action) {
	int ret = 0;

	switch (action) {
		case tape_quit:
		case tape_menu:
			current_tcount = 1;
			current_text[0] = "quit menu";
			ret = 1;
			break;
		case tape_stop:
			if (menu_item[idx].parent == M_ROOT) {
				current_tcount = 1;
				current_text[0] = "stop menu";
				ret = 1;
			} else {
				idx = up(idx);
			}
			break;
		case tape_play:
			if (menu_item[idx].func) {
				current_tcount = 1;
				current_text[0] = (*menu_item[idx].func)(&menu_item[idx]);
				current_text[0] = menu_item[idx].command;
				ret = 2;
			} else {
				idx = down(idx);
			}
			break;
		case tape_track_next:
		case tape_track_prev:
			idx = next(idx, action == tape_track_next);
			break;
	}

	return ret;
}

void CMenu::menulist(int outdev, int indent, menu_id root_id) {
	int i,j;

	i = 0;
	while (i<SIZE_MENU) {
		if (menu_item[i].parent == root_id) {
			j = indent * 3;
			while (j--) {
				dprintf(outdev, " ");
			}
			dprintf(outdev, "%c", menu_item[i].func ? '#' : '*' );
			dprintf(outdev, " [%-7s] - %s\n", menu_item[i].name, menu_item[i].desc);
			menulist(outdev, indent+1, menu_item[i].id);
		}
		i++;
	}
}

void CMenu::help(int outdev) {
	
	dprintf(outdev, "Menu\n");
	menulist(outdev, 1, M_ROOT);
	dprintf(outdev, "\nKeys in menu:\n");
	dprintf(outdev, "\t[Next] Next menu item.\n");
	dprintf(outdev, "\t[Prev] Previous menu item.\n");
	dprintf(outdev, "\t[Play] One level down or execute command.\n");
	dprintf(outdev, "\t[Stop] One level up or return to recording.\n");
	dprintf(outdev, "\t[Menu] Return to recording.\n");
	dprintf(outdev, "\n");
}

#ifndef HAVE_CONFIG_H

int main(void) {
	CMenu *menu;
	int i;
	int tc;
	char **tt;
	
	menu = new CMenu("1234.wav");
	menu->help(stdout);

	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	//menu->do_action(key_play, &tc, tt);
	menu->do_action(key_track_next, &tc, tt);
	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	menu->do_action(key_track_next, &tc, tt);
	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	menu->do_action(key_play, &tc, tt);
	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	menu->do_action(key_track_next, &tc, tt);
	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	menu->do_action(key_play, &tc, tt);
	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	delete menu;

	return 0;
}
#endif

==> speakrec-0.1.6.tar.gz: speakrec-0.1.6/menu.cpp <==
/*
 * SpeakRec - Menu
 * Copyright  2003 Schou Industries ApS.
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "speakrec.h"
#include "menu.h"

/*
 * Define menu function prototype.
 */
#define M_F (char *(*)(struct menuset *menu_entry))

struct menuset {
	menu_id id;
	menu_id parent;
	char *command;
	char *(*func)(struct menuset *menu_entry);
	char *name;
	char *desc;
};

char *currentname(void) {
	return "1234";
}

char system_command[256];

extern int opt_lf30hz;
extern off_t counter_index;
extern off_t counter_size;

const char *menu_lf30hz[] = {
	"Confirm turn lf30hz on",
	"Confirm turn lf30hz off"
};

char *act_rstcnt(struct menuset *menu_entry) {

	counter_index = counter_size;
	menu_entry->desc = (char *)"Reset counter";
	menu_entry->name = (char *)"Rst Cnt";

	return 0;
}

char *act_lf30hz(struct menuset *menu_entry) {

	opt_lf30hz++;
	opt_lf30hz %= 2;
	menu_entry->desc = (char *)menu_lf30hz[opt_lf30hz];
	menu_entry->name = (char *)(opt_lf30hz ? "Cnf off" : "Cnf  on");

	return 0;
}

char *act_system(struct menuset *menu_entry) {
	int ret;

	snprintf(system_command, sizeof(system_command)-1, "%s.%s", PACKAGE, menu_entry->command);
	ret = system(system_command);
	snprintf((char *)&system_command[strlen(system_command)],
			sizeof(system_command)-strlen(system_command),
			" = %d (return code on execute)", ret);

	return system_command;
}

char *act_delall(struct menuset *menu_entry) {
	counter_index = 0;
	return act_system(menu_entry);
}

struct menuset menu_item[] = {
/*
 * id            parent    command   func            name       desc
 */
{ M_RSTCNT,      M_ROOT,   "rstcnt", M_F act_rstcnt, "RST CNT", "Reset time counter" },
{ M_DELALL,      M_ROOT,   NULL,     NULL,           "Del All", "Delete all files" },
//{ M_DELALL_CONF, M_DELALL, "delall", M_F act_system, "Confirm", "Confirm delete all files" },
{ M_DELALL_CONF, M_DELALL, "delall", M_F act_delall, "Confirm", "Confirm delete all files" },
/*
{ M_NET,         M_ROOT,   NULL,     NULL,           "  Net",   "Upload files to server over net" },
{ M_NET_CONF,    M_NET,    "net",    M_F act_system, "Confirm", "Confirm upload files" },
{ M_CDREC,       M_ROOT,   NULL,     NULL,           "Cd Rec",  "Record (burn) a CD-ROM" },
{ M_CDREC_AUDIO, M_CDREC,  "cdra",   M_F act_system, "Audio",   "Audio (music) CD-ROM (stereo)" },
{ M_CDREC_WAV,   M_CDREC,  "cdrw",   M_F act_system, "Wave",    "Data CD-ROM with wav-files (mono)" },
{ M_CDREC_OGG,   M_CDREC,  "cdro",   M_F act_system, "Ogg Vor", "Data CD-ROM with Ogg/Vorbis-files (mono)" },
{ M_CDREC_SPX,   M_CDREC,  "cdrs",   M_F act_system, "Speex",   "Data CD-ROM with Speex-files (mono)" },
{ M_CDREC_MP3,   M_CDREC,  NULL,     NULL,           "MP3",     "Data CD-ROM with mp3-files - Currently disabled due to patent infringement" },
*/
{ M_COPYRIGHT,   M_ROOT,   "cpyrght",M_F act_system, "Cpyrght", "SpeakRec ver " VERSION " Copyright " YEAR " Schou Industries ApS and Speltron.dk" },
{ M_LF30HZ,      M_ROOT,   NULL,     NULL,           "LF30Hz",  "Toggle LF30Hz on/off" },
{ M_LF30HZ_CONF, M_LF30HZ, "lf30hz", M_F act_lf30hz, "Cnf  on", "Confirm on" },
{ M_REBOOT,      M_ROOT,   NULL,     NULL,           "Reboot",  "Reboot the system and start again" },
{ M_REBOOT_CONF, M_REBOOT, "reboot", M_F act_system, "Confirm", "Confirm rebooting system" },
{ M_HALT,        M_ROOT,   NULL,     NULL,           "Halt",    "Halt the system and turn power off" },
{ M_HALT_CONF,   M_HALT,   "halt",   M_F act_system, "Confirm", "Confirm halting system" }
};

#define SIZE_MENU (unsigned int)((sizeof(menu_item) / sizeof(menu_item[0])))

#define inc_index(i) ((i + 1) % SIZE_MENU)
#define dec_index(i) ((i - 1 + SIZE_MENU) % SIZE_MENU)

CMenu::CMenu(char *newcurrentfile) {

	currentfile = newcurrentfile;
	idx = 0;
	settext(idx);

}

CMenu::~CMenu(void) {
}

int CMenu::settext(int index) {
	menu_id tmp_id;
	char *desc;

	current_tcount = 0;
	current_text[current_tcount++] = menu_item[index].name;
	desc = menu_item[index].desc;
	while (menu_item[index].parent != M_ROOT) {
		tmp_id = menu_item[index].parent;
		while (tmp_id != menu_item[index].id) {
			index = inc_index(index);
		}
		current_text[current_tcount++] = menu_item[index].name;
	}
	current_text[current_tcount++] = desc;
	current_text[current_tcount++] = "Press play for enter";
	current_text[current_tcount++] = "Press menu for exit";

	return index;
}

int CMenu::up(int index) {
	menu_id tmp_id;

	tmp_id = menu_item[index].parent;
	while (tmp_id != menu_item[index].id) {
		index = inc_index(index);
	}
	settext(index);

	return index;
}

int CMenu::down(int index) {
	int i;

	i = inc_index(index);
	while (i != index && menu_item[i].parent != menu_item[index].id) {
		i = inc_index(i);
	}
	settext(i);

	return i;
}

int CMenu::next(int index, int forward) {
	menu_id tmp_id;

	tmp_id = menu_item[index].parent;
	do {
		index = forward ? inc_index(index) : dec_index(index);
	} while (tmp_id != menu_item[index].parent);
	settext(index);
	
	return index;
}

int CMenu::do_action(tape_action action) {
	int ret = 0;

	switch (action) {
		case tape_quit:
		case tape_menu:
			current_tcount = 1;
			current_text[0] = "quit menu";
			ret = 1;
			break;
		case tape_stop:
			if (menu_item[idx].parent == M_ROOT) {
				current_tcount = 1;
				current_text[0] = "stop menu";
				ret = 1;
			} else {
				idx = up(idx);
			}
			break;
		case tape_play:
			if (menu_item[idx].func) {
				current_tcount = 1;
				current_text[0] = (*menu_item[idx].func)(&menu_item[idx]);
				current_text[0] = menu_item[idx].command;
				ret = 2;
			} else {
				idx = down(idx);
			}
			break;
		case tape_track_next:
		case tape_track_prev:
			idx = next(idx, action == tape_track_next);
			break;
		default:
			/* TODO */;
	}

	return ret;
}

void CMenu::menulist(int outdev, int indent, menu_id root_id) {
	unsigned int i,j;

	i = 0;
	while (i<SIZE_MENU) {
		if (menu_item[i].parent == root_id) {
			j = indent * 3;
			while (j--) {
				dprintf(outdev, " ");
			}
			dprintf(outdev, "%c", menu_item[i].func ? '#' : '*' );
			dprintf(outdev, " [%-7s] - %s\n", menu_item[i].name, menu_item[i].desc);
			menulist(outdev, indent+1, menu_item[i].id);
		}
		i++;
	}
}

void CMenu::help(int outdev) {
	
	dprintf(outdev, "Menu\n");
	menulist(outdev, 1, M_ROOT);
	dprintf(outdev, "\nKeys in menu:\n");
	dprintf(outdev, "\t[Next] Next menu item.\n");
	dprintf(outdev, "\t[Prev] Previous menu item.\n");
	dprintf(outdev, "\t[Play] One level down or execute command.\n");
	dprintf(outdev, "\t[Stop] One level up or return to recording.\n");
	dprintf(outdev, "\t[Menu] Return to recording.\n");
	dprintf(outdev, "\n");
}

#ifndef HAVE_CONFIG_H

int main(void) {
	CMenu *menu;
	int i;
	int tc;
	char **tt;
	
	menu = new CMenu("1234.wav");
	menu->help(stdout);

	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	//menu->do_action(key_play, &tc, tt);
	menu->do_action(key_track_next, &tc, tt);
	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	menu->do_action(key_track_next, &tc, tt);
	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	menu->do_action(key_play, &tc, tt);
	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	menu->do_action(key_track_next, &tc, tt);
	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	menu->do_action(key_play, &tc, tt);
	for (i=0; i< menu->current_tcount; i++)
		printf(":: %s\n", menu->current_text[i] );
	printf("--------------\n");

	delete menu;

	return 0;
}
#endif
